//import libraries
import { system, world } from "@minecraft/server";
import * as Home from "./myhome.js";
import { ForceShow } from "./ForceShow.js";
import { SBDB } from "./db.js";
import { SIGNAL } from "./dz-signal";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";

//--------------------------- global var
let commandPrefix = "-h";
const chatPrefix = "§8[§bSHome§8]§r ";
const commandList = [
  "move",
  "add",
  "warp",
  "delete",
  "location",
  "change",
  "set",
  "go",
  "remove",
  "del",
  "pos",
  "spawn",
  "setspawn",
];
const commandListNoArg = [
  "help",
  "max",
  "owned",
  "list",
  "clear",
  "yes",
  "no",
  "wait",
];
const db = new SBDB();
const version = "4.7";
let waitTime = 0;

//--------------------------- event
//run saat ada chat
world.beforeEvents.chatSend.subscribe((msg) => {
  //main command
  if (msg.message.startsWith(commandPrefix)) {
    sHome(msg);
  }
});

SIGNAL.connect("setting", (eventData) => {
  if (eventData.isi !== "reload") return;
  db.reload();
  loadData();
});

function loadData() {
  if (!db.has("tp_waitTime")) db.set("tp_waitTime", "0");
  waitTime = parseInt(db.get("tp_waitTime"));
}
loadData();

function sHome(msg) {
  //init sequence
  let srcPlayer = msg.sender;

  let args_ = msg.message.slice(commandPrefix.length).trim().split(" "); //ambil(grap) command and arguments
  let command = args_.shift().toLowerCase(); //ambil command
  let args = args_.join("_").toLowerCase(); //ambil arguments

  msg.cancel = true; //cancel chat
  run(async () => {
    let runHome = new Home.default(srcPlayer);
    let chatKebukak = true;
    //test command name
    if (
      command.length <= 0 ||
      !(commandList.includes(command) || commandListNoArg.includes(command)) ||
      ["ui", "gui"].includes(command)
    ) {
      await srcPlayer.sendMessage({
        rawtext: [{ text: chatPrefix + "§ePlease close chat ui to continue" }],
      });
      command = await bukaCommand(srcPlayer);
      if (command === "cancel") return;
      chatKebukak = false;
    }
    //test apa ini command yg butuh argument || test need argumet or not

    if (commandList.includes(command) && args.length <= 0) {
      if (chatKebukak) {
        await srcPlayer.sendMessage({
          rawtext: [
            { text: chatPrefix + "§ePlease close chat ui to continue" },
          ],
        });
      }
      if (["set", "add"].includes(command)) args = await setHomeName(srcPlayer);
      else args = await getHomeName(srcPlayer, runHome);
      if (args === "raonoisinecuk96" || args.trim() === "")
        return srcPlayer.sendMessage({
          rawtext: [{ text: chatPrefix + `§cneed a home name` }],
        }); //lempar eror
    }

    //blok main command
    switch (command) {
      case "help":
        const helpMsg = `§1>------------4.0------------<\n§3${commandPrefix} <change;set;go;remove;pos;setspawn> <home name>\n§3${commandPrefix} <move;add;warp;delete;location;spawn> <home name>\n§3channge/move §9= to change your home point\n§3set/add §9= to set your home point\n§3go/warp §9= teleport to your home point\n§3remove/delete §9= remove your home point\n§3pos/location §9= show your home point coordinate\n§3setspawn/spawn §9= change your spawn point to your home point\n§1>---------------------------<\n§3${commandPrefix} <list;clear>\n§3list §9= show all home you have\n§3clear §9= remove all home you have\n§1>---------------------------<\n§9space in <home name> will be replace by '_'\n§1>---------------------------<`;
        srcPlayer.sendMessage({ rawtext: [{ text: `${helpMsg}` }] });
        if (srcPlayer.hasTag("rank:mod") || srcPlayer.isOp())
          srcPlayer.sendMessage({
            rawtext: [
              {
                text: `§3${commandPrefix} max [number] §9= see or set max home\n§3${commandPrefix} wait [number] §9= see or set wait time`,
              },
            ],
          });
        break;
      case "max":
        if (srcPlayer.hasTag("rank:mod") || srcPlayer.isOp())
          run(() => runHome.limit(args));
        break;
      case "wait":
        if (srcPlayer.hasTag("rank:mod") || srcPlayer.isOp())
          run(() => {
            const isnum = parseInt(args);
            if (!isNaN(isnum) && isnum >= 0) {
              db.set("tp_waitTime", isnum.toString());
              waitTime = isnum;
            }
            srcPlayer.sendMessage(`§atp wait time is set to §6${waitTime}`);
          });
        break;
      case "add":
      case "set":
        run(() => runHome.addHome(args));
        break;
      case "move":
      case "change":
        run(() => runHome.change(args));
        break;
      case "warp":
      case "go":
        if (srcPlayer.hasTag("mc:incombat"))
          return srcPlayer.sendMessage(
            `§cyou can't go to your home now, you are in combat`,
          );
        run(() => runHome.goHome(args, waitTime));
        break;
      case "delete":
      case "del":
      case "remove":
        run(() => runHome.removeHome(args));
        break;
      case "owned":
      case "list":
        runHome.list();
        break;
      case "spawn":
      case "setspawn":
        run(() => runHome.setSpawn(args));
        break;
      case "pos":
      case "position":
      case "location":
        runHome.location(args);
        break;
      case "clear":
        run(() => runHome.clearStart());
        break;
      case "yes":
        run(() => runHome.clearYes());
        break;
      case "no":
        run(() => runHome.clearNo());
        break;
      default:
        srcPlayer.sendMessage({
          rawtext: [
            {
              text:
                chatPrefix +
                `§cunknown command, type "-h help" to see all command available`,
            },
          ],
        });
        break;
    }
  });
}

world.afterEvents.playerSpawn.subscribe((data) => {
  if (!data.initialSpawn) return;
  if (data.player.hasTag("§d§z§c§l§e§ar_§h§o§m§e"))
    data.player.removeTag("§d§z§c§l§e§ar_§h§o§m§e");
  data.player.sendMessage({
    rawtext: [{ text: `§2+ §aSHome v${version} is installed` }],
  });
});

function run(func) {
  system.run(func);
}
// --------------------------- gui
async function bukaCommand(player) {
  let actionForm = new ActionFormData();
  actionForm.title("SHome GUI");
  actionForm.body(`What do you want to do?\n§jdo "-h help" for more info`);
  actionForm.button("§fSet\n§jadd/set new home", "textures/ui/color_plus");
  actionForm.button(
    "§fChange\n§jchange/move your home",
    "textures/ui/book_edit_default",
  );
  actionForm.button("§fGo\n§jtp to your home", "textures/ui/icon_import");
  actionForm.button("§fDelete\n§jremove a house", "textures/ui/icon_trash");
  actionForm.button("§fList\n§jsee all your home", "textures/ui/creative_icon");
  actionForm.button(
    "§fSpawn\n§jset home as spawn poin",
    "textures/items/bed_red",
  );
  actionForm.button("§fPos\n§jsee xyz of a house", "textures/ui/xyz_axis");

  actionForm.button("§4Cancel", "textures/ui/cancel");
  const actionFormResponse = await ForceShow(actionForm, player);
  const { canceled, selection, cancelationReason } = actionFormResponse;
  if (canceled === true) {
    if (cancelationReason == "userBusy")
      player.sendMessage({
        rawtext: [
          {
            text: chatPrefix + "can't open ui, please close other ui first",
          },
        ],
      });
    return "cancel";
  }
  const _r = [
    "add",
    "move",
    "go",
    "remove",
    "list",
    "setspawn",
    "location",
    "cancel",
  ];
  return _r[selection];
}

async function setHomeName(player) {
  let modalForm = new ModalFormData();
  modalForm.title("Home Name");
  modalForm.textField(
    "\nSet Your Home Name\n§8- only text\n§8- lower case\n§8- no underscores",
    "",
  );
  const modalFormResponse = await ForceShow(modalForm, player);
  const { formValues, canceled } = modalFormResponse;
  if (canceled) return "raonoisinecuk96";
  const jeneng = formValues;
  return jeneng?.join("_")?.toLowerCase() || "raonoisinecuk96";
}

async function getHomeName(player, data) {
  const hname = data.getHomeName();
  if (hname.length <= 0) return "raonoisinecuk96";

  let actionForm = new ActionFormData();
  actionForm.title("SHome GUI");
  actionForm.body(`what home?`);
  for (const prop in hname) {
    actionForm.button(hname[prop], "textures/ui/magnifyingGlass");
  }
  actionForm.button("§4Cancel", "textures/ui/cancel");
  const actionFormResponse = await ForceShow(actionForm, player);
  const { canceled, selection, cancelationReason } = actionFormResponse;
  if (canceled === true) {
    if (cancelationReason == "userBusy")
      player.sendMessage({
        rawtext: [
          {
            text: chatPrefix + "can't open ui, please close other ui first",
          },
        ],
      });
    return "raonoisinecuk96";
  }
  if (selection === hname.length) return "raonoisinecuk96";
  return hname[selection];
}
