//import data library
import { world, MolangVariableMap } from "@minecraft/server";
import { DIMENSION } from "./dimension.js";
import { waitTimer } from "./waitTimer.js";

export default class Home {
  constructor(srcPlayer) {
    this.srcPlayer = srcPlayer;
    this.player = srcPlayer.name ?? srcPlayer.nameTag;
    this.getHomeTag = srcPlayer.getTags().join(", ");
    this.allHomeName = this.extractHomeNames(this.getHomeTag) || [];
    this.playerDimension = currentDimension(srcPlayer);
    this.max_home = world.getDynamicProperty("dz_homeLimit") || Infinity;
  }

  // Ekstrak semua nama "home" menggunakan regex
  extractHomeNames(tag) {
    const homeNamesReg = /(?<=§4H_).+?(?=§kD_)/g;
    return tag.match(homeNamesReg);
  }

  // Ambil posisi dan rotasi pemain
  playerPos() {
    const location = this.srcPlayer.location;
    const rotation = this.srcPlayer.getRotation();
    this.playerX = Math.floor(location.x);
    this.playerY = Math.floor(location.y);
    this.playerZ = Math.floor(location.z);
    this.playerRotX = rotation.x.toFixed(3);
    this.playerRotY = rotation.y.toFixed(3);
    this.playerDimId = dimToId(this.playerDimension);
  }

  // Atur batas jumlah "home"
  limit(args) {
    const isnum = parseInt(args);
    if (!isNaN(isnum) && isnum >= 0) {
      world.setDynamicProperty("dz_homeLimit", isnum);
      this.srcPlayer.sendMessage(
        `§amax home is set to §6${isnum === 0 ? "Infinity" : isnum}`,
      );
    } else {
      this.srcPlayer.sendMessage(`§amax home is set to §6${this.max_home}`);
    }
  }

  // Set UI hurt mode
  uihurt(args) {
    const isnum =
      args === "true" || args === "1"
        ? true
        : args === "false" || args === "0"
          ? false
          : world.getDynamicProperty("dz_homeUiHurt");
    world.setDynamicProperty("dz_homeUiHurt", isnum);
    this.srcPlayer.sendMessage(`§agui hurt is set to §6${isnum}`);
  }

  // Ambil informasi posisi home berdasarkan nama
  homePos(args) {
    const homeReg = new RegExp(
      `§4H_${args}§kD_(\\d+)X_(-?\\d+)Y_(-?\\d+)Z_(-?\\d+)RX_(-?\\d+\\.\\d+)RY_(-?\\d+\\.\\d+)§r`,
    );
    const homeFind = this.getHomeTag.match(homeReg);
    if (!homeFind) return null;

    const [_, dim, x, y, z, rotX, rotY] = homeFind;
    return {
      name: args,
      x: parseInt(x),
      y: parseInt(y),
      z: parseInt(z),
      rotX: parseFloat(rotX),
      rotY: parseFloat(rotY),
      dim: idToDim(dim),
    };
  }
  // Ambil informasi posisi home berdasarkan nama tanpa parse
  homePosNo(args) {
    const homeReg = new RegExp(
      `§4H_${args}§kD_(\\d+)X_(-?\\d+)Y_(-?\\d+)Z_(-?\\d+)RX_(-?\\d+\\.\\d+)RY_(-?\\d+\\.\\d+)§r`,
    );
    const homeFind = this.getHomeTag.match(homeReg);
    if (!homeFind) return null;

    const [_, dim, x, y, z, rotX, rotY] = homeFind;
    return {
      name: args,
      x,
      y,
      z,
      rotX,
      rotY,
      dim: idToDim(dim),
    };
  }

  // Tambahkan "home" baru
  addHome(args) {
    if (this.allHomeName.length >= this.max_home) {
      this.srcPlayer.sendMessage(
        `§cyou already have §5${this.max_home}§c homes.`,
      );
    } else if (this.allHomeName.includes(args)) {
      this.srcPlayer.sendMessage(
        `§cyou already have a home called §5${args}§c.`,
      );
    } else {
      this.playerPos();
      this.srcPlayer.addTag(
        `§4H_${args}§kD_${this.playerDimId}X_${this.playerX}Y_${this.playerY}Z_${this.playerZ}RX_${this.playerRotX}RY_${this.playerRotY}§r`,
      );
      this.srcPlayer.dimension.spawnParticle(
        "minecraft:crop_growth_emitter",
        {
          x: this.playerX + 0.5,
          y: this.playerY + 0.15,
          z: this.playerZ + 0.5,
        },
        new MolangVariableMap(),
      );
      this.srcPlayer.sendMessage(
        `§ayou set a home named §5${args}§a at X:§5${this.playerX}§a, Y:§5${this.playerY}§a, Z:§5${this.playerZ}`,
      );
    }
  }

  // Ubah lokasi "home" yang ada
  change(args) {
    if (!this.allHomeName.includes(args)) {
      this.srcPlayer.sendMessage(`§cyou don't have a home named §5${args}§c.`);
      return;
    }
    const home = this.homePosNo(args);
    if (!home) return;

    this.playerPos();
    this.srcPlayer.removeTag(
      `§4H_${args}§kD_${dimToId(home.dim)}X_${home.x}Y_${home.y}Z_${home.z}RX_${home.rotX}RY_${home.rotY}§r`,
    );
    this.srcPlayer.addTag(
      `§4H_${args}§kD_${this.playerDimId}X_${this.playerX}Y_${this.playerY}Z_${this.playerZ}RX_${this.playerRotX}RY_${this.playerRotY}§r`,
    );
    this.srcPlayer.dimension.spawnParticle(
      "minecraft:crop_growth_emitter",
      { x: this.playerX + 0.5, y: this.playerY + 0.15, z: this.playerZ + 0.5 },
      new MolangVariableMap(),
    );
    this.srcPlayer.sendMessage(`§ayou changed your home §5${args} §alocation.`);
  }

  // Pergi ke "home" berdasarkan nama
  goHome(args, waitTime) {
    if (!this.allHomeName.includes(args)) {
      this.srcPlayer.sendMessage(`§cyou don't have a home named §5${args}§c.`);
      return;
    }
    const home = this.homePos(args);
    if (!home) return;

    const monstersNearby = [
      ...this.srcPlayer.dimension.getEntities({
        location: this.srcPlayer.location,
        maxDistance: 8,
        families: ["monster"],
      }),
    ];
    if (monstersNearby.length > 0) {
      this.srcPlayer.sendMessage(
        `§cThere are monsters nearby, please move to a safe place first.`,
      );
      return;
    }
    this.playerPos();
    waitTimer(this.srcPlayer, waitTime, () => {
      this.srcPlayer.dimension.spawnParticle(
        "minecraft:egg_destroy_emitter",
        {
          x: this.playerX + 0.5,
          y: this.playerY + 0.15,
          z: this.playerZ + 0.5,
        },
        new MolangVariableMap(),
      );
      this.srcPlayer.teleport(
        { x: home.x + 0.5, y: home.y + 0.25, z: home.z + 0.5 },
        {
          dimension: world.getDimension(home.dim),
          rotation: { x: home.rotX, y: home.rotY },
        },
      );
      this.srcPlayer.playSound("mob.endermen.portal", {
        volume: 0.48,
        pitch: 1.3,
      });
      this.srcPlayer.sendMessage(`§ayou were teleported to home §5${args}`);
    });
  }

  // Hapus "home"
  removeHome(args) {
    if (!this.allHomeName.includes(args)) {
      this.srcPlayer.sendMessage(`§cyou don't have a home named §5${args}§c.`);
      return;
    }
    const home = this.homePosNo(args);
    if (!home) return;
    console.warn(JSON.stringify(home));
    console.warn(
      `§4H_${args}§9D_${dimToId(home.dim)}X_${home.x}Y_${home.y}Z_${home.z}RX_${home.rotX}RY_${home.rotY}§r`,
    );
    console.warn(this.getHomeTag.replace("§k", "§9"));

    this.srcPlayer.removeTag(
      `§4H_${args}§kD_${dimToId(home.dim)}X_${home.x}Y_${home.y}Z_${home.z}RX_${home.rotX}RY_${home.rotY}§r`,
    );
    this.srcPlayer.sendMessage(`§ayou removed home §5${args}`);
  }

  // List semua "home"
  list() {
    if (this.allHomeName.length < 1) {
      this.srcPlayer.sendMessage(`§cno homes found.`);
    } else {
      this.srcPlayer.sendMessage(
        `§aYour homes: §5${this.allHomeName.join(", ")}\n§aHome slots: ${this.allHomeName.length}/${this.max_home}`,
      );
    }
  }

  // Set spawn point ke posisi home
  setSpawn(args) {
    if (!this.allHomeName.includes(args)) {
      this.srcPlayer.sendMessage(`§cyou don't have a home named §5${args}§c.`);
      return;
    }

    const home = this.homePos(args);
    if (!home) return;

    this.srcPlayer.setSpawnPoint({
      x: home.x,
      y: home.y,
      z: home.z,
      dimension: DIMENSION[home.dim],
    });

    try {
      // Spawn beberapa partikel di lokasi spawn point
      for (let i = 0; i < 4; i++) {
        DIMENSION[home.dim].spawnParticle(
          "minecraft:balloon_gas_particle",
          {
            x: home.x + 0.5,
            y: home.y + (0.15 + i * 0.5),
            z: home.z + 0.5,
          },
          new MolangVariableMap(),
        );
      }
    } catch (e) {
      return;
    }

    this.srcPlayer.sendMessage(
      `§aYou set your spawn at home §5${args} §a(X:§5${home.x}§a, Y:§5${home.y}§a, Z:§5${home.z}§a)`,
    );
  }

  // Tampilkan lokasi home ke pemain
  location(args) {
    if (!this.allHomeName.includes(args)) {
      this.srcPlayer.sendMessage(`§cyou don't have a home named §5${args}§c.`);
      return;
    }

    const home = this.homePos(args);
    if (!home) return;

    this.srcPlayer.sendMessage(
      `§aHome §5${args} §ais located at X:§5${home.x}§a, Y:§5${home.y}§a, Z:§5${home.z}§a in the dimension: §5${home.dim}`,
    );
  }

  // Clear semua home dengan konfirmasi
  clearStart() {
    if (this.allHomeName.length < 1) {
      this.srcPlayer.sendMessage(`§cNo homes found.`);
      return;
    }

    this.srcPlayer.addTag("§d§z§c§l§e§ar_§h§o§m§e");
    this.srcPlayer.sendMessage(
      `§aAre you sure you want to clear all your homes? Type §5-h yes §ato confirm, or §5-h no §ato cancel.`,
    );
  }

  // Konfirmasi clear semua home
  clearYes() {
    if (!this.srcPlayer.hasTag("§d§z§c§l§e§ar_§h§o§m§e")) {
      this.srcPlayer.sendMessage(`§cCan't use this command.`);
      return;
    }

    for (const homeName of this.allHomeName) {
      const home = this.homePosNo(homeName);
      if (home) {
        this.srcPlayer.removeTag(
          `§4H_${home.name}§kD_${dimToId(home.dim)}X_${home.x}Y_${home.y}Z_${home.z}RX_${home.rotX}RY_${home.rotY}§r`,
        );
      }
    }

    this.srcPlayer.removeTag("§d§z§c§l§e§ar_§h§o§m§e");
    this.srcPlayer.sendMessage(`§aAll homes have been removed.`);
  }

  // Batalkan clear home
  clearNo() {
    if (this.srcPlayer.hasTag("§d§z§c§l§e§ar_§h§o§m§e")) {
      this.srcPlayer.removeTag("§d§z§c§l§e§ar_§h§o§m§e");
      this.srcPlayer.sendMessage(`§aOperation canceled.`);
    } else {
      this.srcPlayer.sendMessage(`§cCan't use this command.`);
    }
  }

  // Dapatkan nama-nama home pemain
  getHomeName() {
    return this.allHomeName;
  }
}

// Fungsi pembantu
function currentDimension(player) {
  return player.dimension.id.replace("minecraft:", "");
}

function dimToId(dimension) {
  return dimension === "overworld"
    ? "0"
    : dimension === "nether"
      ? "1"
      : dimension === "the_end"
        ? "2"
        : "0";
}

function idToDim(id) {
  return id === "0"
    ? "overworld"
    : id === "1"
      ? "nether"
      : id === "2"
        ? "the_end"
        : "overworld";
}
